@@ -5,6 +5,7 @@ import warnings
 
 warnings.filterwarnings("ignore")  # 忽略警告
 
+
 class eamonn(object):
     # ===============请求类=========================================
     @staticmethod  # 请求器
@@ -277,7 +278,7 @@ class eamonn(object):
         return data
 
     @staticmethod  # 图片下载器
-    def u2f(url, path=None, headers="",save=True):
+    def u2f(url, path=None, headers="", save=True):
         import requests
         if headers == "":
             header_s = {"User-Agent": eamonn.rua()}
@@ -539,8 +540,6 @@ class eamonn(object):
             inst[k] = eamonn.d2o(v)
         return inst
 
-
-
     @staticmethod  # 数组切片
     def cut(arr, cp=20):
         arc, tmp_arr, tmp_a = [], [], []
@@ -593,7 +592,7 @@ class eamonn(object):
         manager = Manager()
         q = manager.Queue()
         lock = manager.Lock()
-        return q,lock
+        return q, lock
 
     @staticmethod  # gevent协程
     def gPool(callback, urls=[], pnum=800):
@@ -665,8 +664,8 @@ class eamonn(object):
         return Queue()
 
     @staticmethod
-    def page(total,num):
-        return (int(total)+int(num)-1)//int(num)
+    def page(total, num):
+        return (int(total) + int(num) - 1) // int(num)
 
     @staticmethod
     def init():
@@ -679,15 +678,15 @@ class eamonn(object):
         os.chdir(path)  # 修改工作目录
 
     @staticmethod
-    def open_pict(path,byte=False):
+    def open_pict(path, byte=False):
         from PIL import Image
-        img=Image.open(path)
+        img = Image.open(path)
         if byte:
             return eamonn.image_to_byte(img)
         return img
 
     # 图片转bytes
-    @staticmethod   
+    @staticmethod
     def image_to_byte(image):
         import io
         imgByteArr = io.BytesIO()
@@ -696,13 +695,14 @@ class eamonn(object):
         return imgByteArr
 
     @staticmethod
-    def change_pict(from_path,go_path):
+    def change_pict(from_path, go_path):
         from PIL import Image
         from io import BytesIO
-        if type(from_path)==str:
+        if type(from_path) == str:
             im = Image.open(from_path)
         else:
-            im=from_path
+            im = from_path
+
         def iter_frames(im):
             try:
                 i = 0
@@ -717,6 +717,7 @@ class eamonn(object):
                     i += 1
             except EOFError:
                 pass
+
         for i, frame in enumerate(iter_frames(im)):
             with BytesIO() as output:
                 with frame as img:
@@ -728,22 +729,61 @@ class eamonn(object):
 
     #  转存
     @staticmethod
-    def save_pict(from_path,go_path):
+    def save_pict(from_path, go_path):
         from PIL import Image
-        img=eamonn.image_to_byte(Image.open(from_path,mode='r'))
+        img = eamonn.image_to_byte(Image.open(from_path, mode='r'))
         # print(img)
         eamonn.rw(go_path, img, aw='wb')
 
+    @staticmethod
+    def save_excel(resList, excel_name='data', title=None, path=""):
+        """
+        将列表数据保存到表格
+        :param resList:
+        :param excel_name:
+        :param title:
+        :param path:
+        :return:
+        """
+        import xlsxwriter
+        from tqdm import tqdm
+
+        file_name = f"{excel_name}.xlsx"
+        if path:
+            ezfuc.make_dir(path=path)
+            file_name = path + file_name
+        workbook = xlsxwriter.Workbook(file_name)
+        sheet = workbook.add_worksheet('Sheet1')  # 创建一个sheet表格
+
+        l = 0
+        if not isinstance(resList[-1], list):
+            try:
+                resList = [list(i.values()) for i in resList]
+            except AttributeError:
+                resList = [[i] for i in resList]
+        if title:
+            resList.insert(0, title)
+        for line in tqdm(resList, ascii=True, desc=excel_name):
+            r = 00
+            for i in line:
+                try:
+                    sheet.write(l, r, i.strip())  # 一个一个将单元格数据写入
+                except AttributeError:
+                    sheet.write(l, r, i)
+                r = r + 1
+            l = l + 1
+        workbook.close()
+
     # 图片灰度
     @staticmethod
-    def gray_pict(from_path,go_path,lay=""):
+    def gray_pict(from_path, go_path, lay=""):
         from PIL import Image
         from io import BytesIO
-        if type(from_path)==str:
+        if type(from_path) == str:
             img = Image.open(from_path, mode="r")
             Img = img.convert('L')
         else:
-            Img=from_path
+            Img = from_path
         with BytesIO() as output:
             with Img as img:
                 img.save(output, "gif")
@@ -751,17 +791,16 @@ class eamonn(object):
             eamonn.rw(go_path, data, aw='wb')
             eamonn.change_pict(from_path=go_path, go_path=go_path)
 
-
     # 图片二值化
     @staticmethod
-    def ib_pict(from_path,go_path,threshold=200,lay="",form=""):
+    def ib_pict(from_path, go_path, threshold=200, lay="", form=""):
         from PIL import Image
         from io import BytesIO
-        if type(from_path)==str:
+        if type(from_path) == str:
             img = Image.open(from_path, mode="r")
             Img = img.convert('L')
         else:
-            Img=from_path
+            Img = from_path
         # 自定义灰度界限，大于这个值为黑色，小于这个值为白色
         table = []
         for i in range(256):
@@ -778,7 +817,6 @@ class eamonn(object):
             eamonn.rw(go_path, data, aw='wb')
             eamonn.change_pict(from_path=go_path, go_path=go_path)
 
-
     # ===============pythonic 修饰器=========================================
     @staticmethod
     def exp():
@@ -791,7 +829,9 @@ class eamonn(object):
                 t = 0.1 if t == 0.0 else t
                 print('call %s %.4fs' % (func.__name__, t))
                 return res
+
             return wrapper
+
         return deco
 
     @staticmethod
@@ -920,7 +960,7 @@ class eamonn(object):
             # if i==6:return time.strftime("%Y", time.localtime())
             if i == 8: return datetime.datetime.now()
             if i == 9: return round(time.time())
-            if i ==10: return ""
+            if i == 10: return ""
 
         else:
             if st == '' and oper == '':
@@ -956,7 +996,6 @@ class eamonn(object):
             return True
         return False
 
-
     # ===============数据库操作类=========================================
     @staticmethod
     class save_mongo():
@@ -964,7 +1003,7 @@ class eamonn(object):
             import pymongo
             if fname == '':
                 if time:
-                    self.cname = 'Default_name'+"_" + eamonn.time(2)
+                    self.cname = 'Default_name' + "_" + eamonn.time(2)
                 else:
                     self.cname = 'Default_name'
             else:
@@ -983,7 +1022,7 @@ class eamonn(object):
             self.collection.insert_many(arr)
 
     @staticmethod
-    def get_mongo(ip="127.0.0.1", dname="", fname="",username="",password="",Chart=True):
+    def get_mongo(ip="127.0.0.1", dname="", fname="", username="", password="", Chart=True):
         import pymongo
         assert dname, "dname 不能为空"
         if Chart:
@@ -1003,7 +1042,7 @@ class eamonn(object):
         return db
 
     @staticmethod
-    def save_xlsx(dname, fname,path=""):
+    def save_xlsx(dname, fname, path=""):
         from eamonn import eamonn
         from openpyxl.workbook import Workbook
         from openpyxl.styles import Font, Alignment, PatternFill, Border, Side
@@ -1033,8 +1072,10 @@ class eamonn(object):
         tasks_list = []
         a = 1
         for i in get_mongo:
-            try:    i.pop("_id")
-            except: pass
+            try:
+                i.pop("_id")
+            except:
+                pass
             tasks_list.append(["" if x == None else x for x in list(i.values())])
             a += 1
         for data in range(1, len(data_list) + 1):
@@ -1195,20 +1236,18 @@ class eamonn(object):
         def close(self, filename):
             self.book.save(filename_or_stream='{}'.format(str(filename)))  # 一定要保存
 
-
-
     @staticmethod
-    def get_xlsx(path,first=1):
+    def get_xlsx(path, first=1):
         import xlrd
-        workbook=xlrd.open_workbook(str(path))
-        sheet_name = workbook.sheet_names()[0]   # 获取所有sheet
+        workbook = xlrd.open_workbook(str(path))
+        sheet_name = workbook.sheet_names()[0]  # 获取所有sheet
         # 根据sheet索引或者名称获取sheet内容
-        sheet = workbook.sheet_by_index(0) # sheet索引从0开始
-        result_list=[]
+        sheet = workbook.sheet_by_index(0)  # sheet索引从0开始
+        result_list = []
         while 1:
             try:
-                rows = sheet.row_values(first) # 获取第2行内容
-                first+=1
+                rows = sheet.row_values(first)  # 获取第2行内容
+                first += 1
                 result_list.append(rows)
             except:
                 break
@@ -1216,7 +1255,6 @@ class eamonn(object):
 
     # ===============排序类=========================================
 
-
     #   冒泡排序
     @staticmethod
     def bubble_sort(alist):
@@ -1253,13 +1291,13 @@ class eamonn(object):
                     left += 1
                 # 此时已找到一个比基准大的书，和一个比基准小的数，将他们互换位置
                 (arr[left], arr[right]) = (arr[right], arr[left])
-     
+
             # 当从两边分别逼近，直到两个位置相等时结束，将左边小的同基准进行交换
             (arr[left], arr[key]) = (arr[key], arr[left])
             # 返回目前基准所在位置的索引
             return left
-     
-        def quicksort(arr, left, right):  
+
+        def quicksort(arr, left, right):
             if left >= right:
                 return
             # 从基准开始分区
@@ -1268,7 +1306,7 @@ class eamonn(object):
             # print(arr)
             quicksort(arr, left, mid - 1)
             quicksort(arr, mid + 1, right)
-     
+
         # 主函数
         n = len(lst)
         if n <= 1:
@@ -1276,149 +1314,144 @@ class eamonn(object):
         quicksort(lst, 0, n - 1)
         return lst
 
-
-
     #   插入排序
     @staticmethod
     def insert_sort(lst):
-        n=len(lst)
-        if n<=1:
+        n = len(lst)
+        if n <= 1:
             return lst
-        for i in range(1,n):
-            j=i
-            target=lst[i]            #每次循环的一个待插入的数
-            while j>0 and target<lst[j-1]:       #比较、后移，给target腾位置
-                lst[j]=lst[j-1]
-                j=j-1
-            lst[j]=target            #把target插到空位
-
+        for i in range(1, n):
+            j = i
+            target = lst[i]  # 每次循环的一个待插入的数
+            while j > 0 and target < lst[j - 1]:  # 比较、后移，给target腾位置
+                lst[j] = lst[j - 1]
+                j = j - 1
+            lst[j] = target  # 把target插到空位
 
     #   希尔排序
     @staticmethod
     def shell_sort(lst):
-        def shellinsert(arr,d):
-            n=len(arr)
-            for i in range(d,n):
-                j=i-d
-                temp=arr[i]             #记录要出入的数
-                while(j>=0 and arr[j]>temp):    #从后向前，找打比其小的数的位置
-                    arr[j+d]=arr[j]                 #向后挪动
-                    j-=d
-                if j!=i-d:
-                    arr[j+d]=temp
-        n=len(lst)
-        if n<=1:
+        def shellinsert(arr, d):
+            n = len(arr)
+            for i in range(d, n):
+                j = i - d
+                temp = arr[i]  # 记录要出入的数
+                while (j >= 0 and arr[j] > temp):  # 从后向前，找打比其小的数的位置
+                    arr[j + d] = arr[j]  # 向后挪动
+                    j -= d
+                if j != i - d:
+                    arr[j + d] = temp
+
+        n = len(lst)
+        if n <= 1:
             return lst
-        d=n//2
-        while d>=1:
-            shellinsert(lst,d)
-            d=d//2
+        d = n // 2
+        while d >= 1:
+            shellinsert(lst, d)
+            d = d // 2
         return lst
 
     #   选择排序
     @staticmethod
     def select_sort(lst):
-        n=len(lst)
-        if n<=1:
+        n = len(lst)
+        if n <= 1:
             return lst
-        for i in range(0,n-1):
-            minIndex=i
-            for j in range(i+1,n):
-                if lst[j]<lst[minIndex]:
-                    minIndex=j
-            if minIndex!=i:
-                (lst[minIndex],lst[i])=(lst[i],lst[minIndex])
+        for i in range(0, n - 1):
+            minIndex = i
+            for j in range(i + 1, n):
+                if lst[j] < lst[minIndex]:
+                    minIndex = j
+            if minIndex != i:
+                (lst[minIndex], lst[i]) = (lst[i], lst[minIndex])
         return lst
 
-
     #   堆排序
     @staticmethod
-    def  heap_sort(lst):
-        def heapadjust(arr,start,end):  #将以start为根节点的堆调整为大顶堆
-            temp=arr[start]
-            son=2*start+1
-            while son<=end:
-                if son<end and arr[son]<arr[son+1]:  #找出左右孩子节点较大的
-                    son+=1
-                if temp>=arr[son]:       #判断是否为大顶堆
+    def heap_sort(lst):
+        def heapadjust(arr, start, end):  # 将以start为根节点的堆调整为大顶堆
+            temp = arr[start]
+            son = 2 * start + 1
+            while son <= end:
+                if son < end and arr[son] < arr[son + 1]:  # 找出左右孩子节点较大的
+                    son += 1
+                if temp >= arr[son]:  # 判断是否为大顶堆
                     break
-                arr[start]=arr[son]     #子节点上移
-                start=son                     #继续向下比较
-                son=2*son+1
-            arr[start]=temp             #将原堆顶插入正确位置
-        n=len(lst)
-        if n<=1:
+                arr[start] = arr[son]  # 子节点上移
+                start = son  # 继续向下比较
+                son = 2 * son + 1
+            arr[start] = temp  # 将原堆顶插入正确位置
+
+        n = len(lst)
+        if n <= 1:
             return lst
-        #建立大顶堆
-        root=n//2-1    #最后一个非叶节点（完全二叉树中）
-        while(root>=0):
-            heapadjust(lst,root,n-1)
-            root-=1
-        #掐掉堆顶后调整堆
-        i=n-1
-        while(i>=0):
-            (lst[0],lst[i])=(lst[i],lst[0])  #将大顶堆堆顶数放到最后
-            heapadjust(lst,0,i-1)    #调整剩余数组成的堆
-            i-=1
+        # 建立大顶堆
+        root = n // 2 - 1  # 最后一个非叶节点（完全二叉树中）
+        while (root >= 0):
+            heapadjust(lst, root, n - 1)
+            root -= 1
+        # 掐掉堆顶后调整堆
+        i = n - 1
+        while (i >= 0):
+            (lst[0], lst[i]) = (lst[i], lst[0])  # 将大顶堆堆顶数放到最后
+            heapadjust(lst, 0, i - 1)  # 调整剩余数组成的堆
+            i -= 1
         return lst
 
-
-
-
     #  归并排序
     @staticmethod
     def merge_sort(lst):
-        #合并左右子序列函数
-        def merge(arr,left,mid,right):
-            temp=[]     #中间数组
-            i=left          #左段子序列起始
-            j=mid+1   #右段子序列起始
-            while i<=mid and j<=right:
-                if arr[i]<=arr[j]:
+        # 合并左右子序列函数
+        def merge(arr, left, mid, right):
+            temp = []  # 中间数组
+            i = left  # 左段子序列起始
+            j = mid + 1  # 右段子序列起始
+            while i <= mid and j <= right:
+                if arr[i] <= arr[j]:
                     temp.append(arr[i])
-                    i+=1
+                    i += 1
                 else:
                     temp.append(arr[j])
-                    j+=1
-            while i<=mid:
+                    j += 1
+            while i <= mid:
                 temp.append(arr[i])
-                i+=1
-            while j<=right:
+                i += 1
+            while j <= right:
                 temp.append(arr[j])
-                j+=1
-            for i in range(left,right+1):    #  !注意这里，不能直接arr=temp,他俩大小都不一定一样
-                arr[i]=temp[i-left]
-        #递归调用归并排序
-        def mSort(arr,left,right):
-            if left>=right:
-                return
-            mid=(left+right)//2
-            mSort(arr,left,mid)
-            mSort(arr,mid+1,right)
-            merge(arr,left,mid,right)
-     
-        n=len(lst)
-        if n<=1:
-            return lst
-        mSort(lst,0,n-1)
-        return lst
+                j += 1
+            for i in range(left, right + 1):  # !注意这里，不能直接arr=temp,他俩大小都不一定一样
+                arr[i] = temp[i - left]
 
+        # 递归调用归并排序
+        def mSort(arr, left, right):
+            if left >= right:
+                return
+            mid = (left + right) // 2
+            mSort(arr, left, mid)
+            mSort(arr, mid + 1, right)
+            merge(arr, left, mid, right)
+
+        n = len(lst)
+        if n <= 1:
+            return lst
+        mSort(lst, 0, n - 1)
+        return lst
 
     #   分组排序   不改变原表结构
     @staticmethod
     def count_sort(lst):
-        n=len(lst)
-        num=max(lst)
-        count=[0]*(num+1)
-        for i in range(0,n):
-            count[lst[i]]+=1
-        arr=[]
-        for i in range(0,num+1):
-            for j in range(0,count[i]):
+        n = len(lst)
+        num = max(lst)
+        count = [0] * (num + 1)
+        for i in range(0, n):
+            count[lst[i]] += 1
+        arr = []
+        for i in range(0, num + 1):
+            for j in range(0, count[i]):
                 arr.append(i)
         return arr
 
 
 if __name__ == '__main__':
     # print(eamonn.time(2))
-    pass
\ No newline at end of file
+    pass
